<?php
/*
Plugin Name: Dashboard Site Notes
Plugin URI: http://innerdvations.com/plugins/admin-dashboard-notes/
Description: Create site notes to appear either on the dashboard or at the top of various admin pages.
Version: 1.0
Author: Ben Irvin
Author URI: http://innerdvations.com/
Tags: instructions, manual, admin, notes, notices, instruction manual
Wordpress URI: http://wordpress.org/extend/plugins/admin-dashboard-site-notes/
License: GPLv3
*/

/*
== TODO == (roughly in order of importance and when they'll get done)
* add config option for choosing what roles can create notes
* add taxonomy positions (edit-tags.php)
* add config option for title of dashboard widget and instruction manual
* Italian translation
* expirations/time ranges
* add locations like menu/widget/settings/comments/etc (maybe just allow entering the filename like options-general.php? that would allow it to work even on plugin pages, etc)
* add note options to appear in areas outside the admin

== Changelog ==
= 1.0 =
* added instruction manual

= 0.9.2.1 =
* fixed fatal error bug and display bug
* fixed incorrect screenshots

= 0.9.2 =
* added translation support and English .pot file
* added donate links
* hierarchy now takes effect on notes on dashboard
* if excerpt is available, use that in notices/dashboard, with full text appearing only in manual
* major optimization by changing note querying method and removing other stupidity
* stored 'everywhere' checkboxes server-side so they work without javascript enabled
* added GPLv3 license document
* improved auto ("everywhere") checkbox functionality

= 0.9.1 =
* notes can be set to display only for certain user roles

= 0.9 =
* added post type notes by action
* added dashboard notes

*/

class DSNManager {
	private static $plugin_id = 'dsnmanager';
	private static $plugin_stylesheet_id = 'dsnmanager_css';
	private static $plugin_script_id = 'dsnmanager_js';
	private static $options_id = 'dsnmanager_options';
	private static $has_edit = false;
	public  static $base;
	public  static $exclude_types = array('nav_menu_item');
	public  static $post_types = array();
	public  static $post_type_name = 'dsn_note';
	public  static $custom_field_prefix = '_dsn_'; // the _ hides it from dropdowns
	public  static $custom; // custom field cache for this post
	private static $instruction_page_title;
	private static $instruction_nav_title;
	
	function __construct() {
		self::$instruction_nav_title = __("Site Instructions");
		self::$instruction_page_title = __("Site Instruction Manual");
		
		self::$has_edit = is_super_admin();
		self::$base = plugin_basename(__FILE__); 
		if(is_admin()) { // if we're in the admin pages...
			load_plugin_textdomain(self::$plugin_id, false, basename( dirname( __FILE__ ) ) . '/languages' );
			
			add_filter('plugin_row_meta',array($this,'extra_plugin_links'),10,2);
			
			// add hooks
			add_action('all_admin_notices',array($this,'all_admin_notices'));
			add_action('init',array($this,'init'), 2);
			add_action('admin_init',array($this,'admin_init'));
			
			// add styles/scripts
			wp_register_style(self::$plugin_stylesheet_id, WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)) . '/admin-styles.css' );
			wp_enqueue_style(self::$plugin_stylesheet_id); 
			wp_register_script(self::$plugin_script_id, WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)) . '/admin-scripts.js' );
			wp_enqueue_script(self::$plugin_script_id); 
			
			// add dashboard notes if there are any
			if(self::has_dashboard_notes()) {
				add_action('wp_dashboard_setup', array($this,'setup_dashboard'));
			}
			// add instruction manual page if entries exist
			if(self::has_instruction_notes()) {
				add_action( 'admin_menu', array($this,'admin_menu') );
			}
		}
	}
	
	public function admin_menu() {
		add_dashboard_page(self::$instruction_page_title, self::$instruction_nav_title, 'read', self::$plugin_id, array($this,'admin_page'));
	}
	public static function admin_page() {
		echo "<div id='dsn_instructions' class='wrap'>";
		echo "<h2>" . self::$instruction_page_title . "</h2>";
		
		$posts = self::get_notes_by_parent(0);
		
		// generate table of contents
		$output = '';
		foreach($posts as $post) {
			$output .= self::index_with_children($post);
		}
		echo "<div class='instruction_index'>";
		echo "<h3>" . __('Table of Contents') . "</h3>";
		echo $output;
		echo "</div>";
		
		// generate instructions
		$output = '';
		foreach($posts as $post) {
			$output .= self::note_with_children($post, 0, true);
		}
		echo "<div class='instruction_content'>";
		echo $output;
		echo "</div>";
		
		echo "</div>";
	}
	
	public static function has_instruction_notes() {
		$args = array('action'=>'instruction','post_parent'=>0);
		$posts = self::get_notes($args);
		if(count($posts)) {
			return true;
		}
		return false;
	}
	
	public static function has_dashboard_notes() {
		$posts = self::get_notes_by_parent(0);
		if(count($posts)) {
			return true;
		}
		return false;
	}
	public function setup_dashboard() {
		wp_add_dashboard_widget('dsn_dashboard' , 'Admin Guide',  array($this,'dsn_dashboard'));
	}
	public static function dsn_dashboard() {
		$posts = self::get_notes_by_parent(0);
		$output = '';
		foreach($posts as $post) {
			$output .= self::note_with_children($post);
		}
		echo $output;
	}
	
	// recursively get the linked post title and it's children
	public static function index_with_children($post,$depth=0) {
		if($depth > 64) { // sanity check
			return "Error: note output aborted, hierarchy depth too deep (>64)";
		}
		$output = "<ul class='index depth-{$depth}'>";
		$id = $post->ID;
		$t = $post->post_title;
		$children = self::get_notes_by_parent($id);
		$child_output = '';
		if(count($children)) {
			foreach($children as $child) {
				$child_output .= self::index_with_children($child,$depth + 1);
			}
		}
		$output .= "
		<li class='dsn dashboard_index depth-{$depth}'>
			<a href='#note_{$id}'><h4>{$t}</h4></a>
			{$child_output}
		</li>";
		$output .= '</ul>';
		return $output;
	}
	// recursively get the post and it's children
	public static function note_with_children($post,$depth=0,$full_post=false) {
		if($depth > 64) { // sanity check
			return "Error: note output aborted, hierarchy depth too deep (>64)";
		}
		$output = "<ul class='notes depth-{$depth}'>";
		$id = $post->ID;
		$t = $post->post_title;
		$c = $post->post_excerpt;
		if(!strlen($c) || $full_post) {
			$c = $post->post_content;
		}
		$c = apply_filters('the_content', $c);
		$children = self::get_notes_by_parent($id);
		$child_output = '';
		if(count($children)) {
			foreach($children as $child) {
				$child_output .= self::note_with_children($child,$depth + 1,$full_post);
			}
		}
		$output .= "
		<li class='dsn site_note depth-{$depth}'>
			<a name='note_{$id}'></a><h4>{$t}</h4>
			<div class='content'>{$c}</div>
			{$child_output}
		</li>";
		$output .= '</ul>';
		return $output;
	}
	
	// returns the current custom post type if applicable, or false if not
	public static function current_post_type() {
		if(isset($_GET['post_type'])) {
			return $_GET['post_type'];
		}
		else if(isset($_GET['post'])) {
			global $post;
			if(isset($post->ID)) {
				return get_post_type($post->ID);
			}
		}
		return '';
	}
	// returns the current action
	public static function current_action() {
		global $pagenow;
		switch($pagenow) {
			case 'index.php':
				if(isset($_GET['page'])) {
					if($_GET['page']==self::$plugin_id) {
						return 'instructions';
					}
					break;
				}
				return 'dashboard';
			case 'edit.php':
				return 'search';
			case 'post-new.php':
				return 'new';
			case 'post.php':
				return 'edit';
		}
		return '';
	}
	public static function get_current_loc($action=null) {
		$on_content_type = self::current_post_type();
		if($action) {
			$on_action = 'instructions';
		}
		else {
			$on_action = self::current_action();
		}
		$loc = "loc_";
		$loc .= $on_action;
		if($on_content_type) {
			$loc .= "_" . $on_content_type;
		}
		return $loc;
	}
	public static function get_everywhere_metakey() {
		return self::$custom_field_prefix . 'loc_all_' . self::current_post_type();
	}
	
	public static function get_notes_by_parent($which_parent=0) {
		$args = array('post_parent'=>(int)$which_parent);
		return self::get_notes($args);
	}
	// return all notes
	public static function get_notes($args=array()) {
		global $wpdb, $wp_roles, $current_user;
		if(!isset($current_user->data->wp_capabilities)) {
			return;
		}
		$post_type_name = self::$post_type_name;
		if(isset($args['action'])) {
			$which_location = self::get_current_loc($args['action']);
		}
		else {
			$which_location = self::get_current_loc();
		}
		// set up the subquery for role checking
		$wheres_arr = array();
		$pre = self::$custom_field_prefix;
		$roles = $current_user->data->wp_capabilities;
		foreach($roles as $role_name=>$role_arr) {
			$role = $wpdb->escape($role_name);
			$wheres_arr[] = " (meta_key = '{$pre}role_{$role}' AND meta_value = '1') ";
		}
		$where_str = implode(" OR ", $wheres_arr);
		if(!strlen($where_str)) {
			return;
		}
		$role_query = " SELECT post_id FROM {$wpdb->postmeta} WHERE {$where_str}";
		
		$post_parent = '';
		if(isset($args['post_parent'])) {
			if(is_array($args['post_parent'])) {
				$p = implode(',',$args['post_parent']);
				$post_parent = " AND {$wpdb->posts}.post_parent IN '{$p}' ";
			}
			else {
				$post_parent = " AND {$wpdb->posts}.post_parent = '{$args['post_parent']}' ";
			}
		}
		
		// build the full query
		if($which_location=='loc_instructions') {
			$sql = $wpdb->prepare("
				SELECT *
				FROM {$wpdb->postmeta}
				LEFT JOIN {$wpdb->posts} ON {$wpdb->posts}.id  = {$wpdb->postmeta}.post_id
				WHERE
					{$wpdb->posts}.post_status = 'publish'
					AND {$wpdb->posts}.post_type = '%s'
					AND	({$wpdb->postmeta}.meta_key = '%s' AND {$wpdb->postmeta}.meta_value = '0')
					AND {$wpdb->posts}.id IN ( {$role_query} )
					{$post_parent}
				GROUP BY {$wpdb->posts}.id
				", $post_type_name, self::$custom_field_prefix . 'instructions_exclude');
		}
		else {
			$sql = $wpdb->prepare("
				SELECT *
				FROM {$wpdb->postmeta}
				LEFT JOIN {$wpdb->posts} ON {$wpdb->posts}.id  = {$wpdb->postmeta}.post_id
				WHERE
					{$wpdb->posts}.post_status = 'publish'
					AND {$wpdb->posts}.post_type = '%s'
					AND	(	
						({$wpdb->postmeta}.meta_key = '%s' AND {$wpdb->postmeta}.meta_value = '1')
						OR
						({$wpdb->postmeta}.meta_key = '%s' AND {$wpdb->postmeta}.meta_value = '1')
						OR
						({$wpdb->postmeta}.meta_key = '{$pre}loc_everywhere' AND {$wpdb->postmeta}.meta_value = '1')
					)
					AND {$wpdb->posts}.id IN ( {$role_query} )
					{$post_parent}
				GROUP BY {$wpdb->posts}.id
				", $post_type_name, self::$custom_field_prefix . $which_location, self::get_everywhere_metakey() );
		}
		$res = $wpdb->get_results($sql);
		return $res;
	}
	
	// Called on hook 'all_admin_notices'
	public function all_admin_notices() {
		// on the dashboard we print a pretty widget, not a notice
		if(self::current_action() == 'dashboard' 
			|| self::current_action() == 'instructions' 
			|| self::current_action() == '') {
			return;
		}
		$posts = self::get_notes();
		$output = '';
		if(count($posts)) {
			foreach($posts as $post) {
				$t = $post->post_title;
				$c = $post->post_content;
				$c = apply_filters('the_content', $c);
				$output .= "<div class='updated dsn'><div class='title'>{$t}</div><div class='content'>{$c}</div></div>";
			}
		}
		echo $output;
	}
	
	// Called on wordpress hook 'admin_init'
	public function admin_init() {
		add_meta_box('display-location-div', __('When and where to display this message'),  array($this,'display_info_metabox'), 'dsn_note', 'normal', 'low');
		add_action('save_post', array($this,'save_meta'));
		$types = get_post_types();
		foreach($types as $type=>$type_obj) {
			if(!in_array($type,self::$exclude_types)) {
				self::$post_types[$type] = get_post_type_object($type);
			}
		}
	}
	
	// Called on wordpress hook 'init'
	public function init() {
		self::add_content_type();
	}
	
	// save all of our meta fields
	public static function save_meta($post_id) {
		// prevent wp from killing our custom fields during autosave
		if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) {
			return $post_id;
		}
		if(is_array($_POST) && count($_POST)) {
			self::check_and_save_checkbox('loc_dashboard',$post_id);
			self::check_and_save_checkbox('instructions_exclude',$post_id);
			self::check_and_save_checkbox('loc_everywhere',$post_id);
			foreach(self::$post_types as $type=>$type_obj) {
				self::check_and_save_checkbox("loc_edit_".$type,$post_id);
				self::check_and_save_checkbox("loc_new_".$type,$post_id);
				self::check_and_save_checkbox("loc_search_".$type,$post_id);
				self::check_and_save_checkbox("loc_all_".$type,$post_id);
			}
			global $wp_roles;
			$roles = $wp_roles->roles;
			foreach($roles as $role_name=>$role_arr) {
				self::check_and_save_checkbox("role_".$role_name,$post_id);
			}
		}
	}
	
	// saves post data from checkboxes
	public static function check_and_save_checkbox($key,$post_id = null) {
		if(!$post_id) {
			global $post;
			$post_id = $post->ID;
		}
		$key = self::$custom_field_prefix . $key;
		if(isset($_POST[$key])) {
			update_post_meta($post_id, $key, 1);
		}
		else {
			update_post_meta($post_id, $key, 0);
		}
	}
	
	public static function get_checkbox($key, $msg, $class='', $is_checked=false) {
		$checked = '';
		$key = self::$custom_field_prefix . $key;
		if(isset(self::$custom[$key][0]) && self::$custom[$key][0] == 1) {
			$checked = " checked='checked' ";
		}
		$ret = "<span class='checkbox'>";
		$ret .= "<input id='{$key}' name='{$key}' class='{$class}' type='checkbox' {$checked} value='{$key}' />";
		$ret .=  "<label for='{$key}'>{$msg}</label>";
		$ret .=  "</span>";
		return $ret;
	}
	public static function display_info_metabox() {
		global $post;
		self::$custom = get_post_custom($post->ID);
		
		echo "<div class='dsn_meta_box'>";
		echo "<div class='meta_item dsn_dashboard'>";
		echo self::get_checkbox("loc_dashboard",__("Include in the dashboard widget"));
		echo "</div>";
		/* TODO: manual action settings
		echo "<div class='dsn_meta_box'>";
		echo "<div class='meta_item dsn_loc_manual'>";
		echo self::get_checkbox("dsn_loc_manual",__("Include this note on the following pages (eg, options-general.php to appear on the Settings->General page)."));
		echo "</div>";
		*/
		echo "<div class='meta_item dsn_instructions_exclude'>";
		echo self::get_checkbox("instructions_exclude",__("Exclude this note from the site instruction manual."));
		echo "</div>";
		
		global $wp_roles;
		$roles = $wp_roles->roles;
		echo "<div class='roles'>";
		echo "<h4>" . __("Show this note for the following roles:") . "</h4>";
        foreach($roles as $role_name=>$role_arr) {
			echo "<div class='role'>";
			echo self::get_checkbox("role_".$role_name,$role_arr['name']);
			echo "</div>";
		}
		echo "</div>";
		echo "<div class='dsn_options'>";
		echo "<h4>" . __("Display this message in the following locations:") . "</h4>";
		
		echo "<table width='100%'>";
		$ct = 0;
		echo "<tr class='even'>
		<td>&nbsp;</td>
		<td>&nbsp;</td>
		<td>&nbsp;</td>
		<td>&nbsp;</td>
		<td>" . self::get_checkbox("loc_everywhere",__("Everywhere"),'master_check') . "</td>
		</tr>";
		foreach(self::$post_types as $type=>$type_obj) {
			if($ct++ % 2 == 0) $class = ' odd ';
			else $class = ' even ';
			echo "<tr class='{$class}'>";
			echo "<td class='label'>{$type_obj->name}:</td>";
			echo "<td class='input {$type_obj->name}'>" . self::get_checkbox("loc_edit_".$type,__("Edit"),'child_check') . "</td>";
			echo "<td class='input {$type_obj->name}'>" . self::get_checkbox("loc_new_".$type,__("New"),'child_check') . "</td>";
			echo "<td class='input {$type_obj->name}'>" . self::get_checkbox("loc_search_".$type,__("Search"),'child_check') . "</td>";
			echo "<td class='input select-all'>" . self::get_checkbox("loc_all_".$type,__("All"),'parent_check',true) . "</td>";
			echo "</tr>";
		}
		echo "</table>";
		echo "</div>";
		
	}
	
	// Create the site note content type
	public static function add_content_type() {
		$labels =  array(
			'name' => __( 'Site Notes' ),
			'singular_name' => __( 'Site Note' ),
			'add_new_item' => __( 'Add Site Note' ),
			'edit_item' => __( 'Edit Site Note' ),
			'new_item' => __( 'New Site Note' ),
			'view_item' => __( 'View Site Note' )
		);
		$supports = array(
			'editor'=>true,
			'title'=>true,
			'page-attributes'=>true,
			'hierarchy'=>true
		);
		// TODO: add capability_type for security so non-superadmins can't add/edit notes
		$args = array(
			'labels' => $labels,
			'public' => false,
			'publicly_queryable' => false,
			'show_ui' => self::$has_edit,
			'hierarchical' => true,
			'page-attributes' => true,
			'revisions' => true,
			'supports' => array('title', 'editor', 'excerpt', 'page-attributes' ),
			'description' => __('Add helpful notes for site admins')			
		);
		register_post_type( self::$post_type_name,$args);
	}
	
	
	// add sickeningly greedy self-serving donate links to the wp plugin page entry
	public static function extra_plugin_links($data, $page) {
		if ( $page == self::$base ) {
			$flattr_url = "http://flattr.com/thing/379485/Dashboard-Site-Notes";
			$paypal_url = "https://www.paypal.com/cgi-bin/webscr?business=donate@innerdvations.com&cmd=_donations&currency_code=EUR&item_name=Donation%%20for%%20Dashboard%%20Site%%20Notes%%20plugin";
			$data = array_merge($data,array(
				sprintf('<a href="%s" target="_blank">%s</a>',$flattr_url, esc_html__('Flattr', self::$plugin_id)),
				sprintf('<a href="%s" target="_blank">%s</a>',$paypal_url, esc_html__('Donate', self::$plugin_id))
			));
		}
		return $data;
	}

}

$dsnmanager = null;
function dsn_init_manager() {
	global $dsnmanager;
	$dsnmanager = new DSNManager();
}
add_action('init','dsn_init_manager', 1);